import logging

from aiogram.dispatcher import FSMContext
from aiogram.dispatcher.filters.state import (
    StatesGroup,
    State,
    default_state
)
from aiogram.types import (
    Message,
    ParseMode,
    ReplyKeyboardMarkup,
    ReplyKeyboardRemove
)

from src.models.crud.server_crud import fn_get_server
from src.models.server import ServerPermissions
from src.settings import settings
from src.utils import chunks
from src.utils.debug_mode import debug_message
from src.utils.decorators import bot_action
from src.utils.server_utils import get_server_by_alias
from src.utils.ssh import session_manager

logger = logging.getLogger(__name__)


class UserStates(StatesGroup):
    initial = State()
    choose_server = State()
    interactive = State()


@bot_action('sys admin interactive')
async def start_interactive_mode(message: Message, state=None):
    available_servers = await fn_get_server(settings.engine, chat_id=str(message.chat.id))
    if not available_servers:
        raise Exception('Cannot start interactive session.\n\nNo server linked to this chat.')
    user_available_servers = list(
        filter(lambda x: x.server_permissions.value >= ServerPermissions.RCE.value or
                         x.server_admin == str(message.from_user.id),
               available_servers)
    )
    if not user_available_servers:
        raise Exception(
            'Cannot start interactive session.\n\nNo server with remote command execution is available for you.')
    await debug_message(message,
                        f'Chat id: {message.chat.id}\n'
                        f'Available rce servers: {list(map(lambda x: x.server_alias, user_available_servers))}\n'
                        f'Current user: {str(message.from_user.id)}\n'
                        )

    await UserStates.initial.set()
    markup = ReplyKeyboardMarkup(resize_keyboard=True, selective=True)
    for server in user_available_servers:
        markup.add(server.server_alias)
    await message.reply("Choose server", reply_markup=markup)


@bot_action(filter_state=UserStates.initial)
async def choose_server(message: Message, state: FSMContext):
    server_alias = message.text
    chosen_server = await get_server_by_alias(message, server_alias)
    await debug_message(message,
                        f'Chat id: {message.chat.id}\n'
                        f'Server permissions: {chosen_server.server_permissions.name} \n'
                        f'Current user: {str(message.from_user.id)}\n'
                        )

    if chosen_server.server_permissions.value < ServerPermissions.RCE.value \
            and str(message.from_user.id) != chosen_server.server_admin:
        raise Exception('You do not have permission for remote code execution.'
                        ' Please ask user who added this server to change permission level.')

    async with state.proxy() as current_state:
        current_state['server_alias'] = chosen_server.server_alias
        current_state['session_id'] = await session_manager.add_connection(chosen_server)
        logger.warning(current_state['session_id'])
    await UserStates.interactive.set()
    await message.reply(
        f'Interactive session created for {server_alias}.\n'
        f'All your messages will be treated as commands, until you send:\n'
        f'`sys admin stop interactive`',
        parse_mode=ParseMode.MARKDOWN,
        reply_markup=ReplyKeyboardRemove())


@bot_action('sys admin stop interactive', filter_state='*')
async def exit_session(message: Message, state: FSMContext):
    if state is None:
        return
    async with state.proxy() as state_proxy:
        state_proxy['server_alias'] = None
        if not ('session_id' in state_proxy and state_proxy['session_id']):
            raise Exception('Session already closed')
        session_manager.close(state_proxy['session_id'])
        state_proxy['session_id'] = None

    await default_state.set()
    await message.reply('Session successfully closed')


@bot_action(filter_state=UserStates.interactive)
async def run_interactive_command(message: Message, state: FSMContext):
    async with state.proxy() as current_state:
        await debug_message(message,
                            f'Chat id: {message.chat.id}\n'
                            f'Current user: {str(message.from_user.id)}\n'
                            f'Session id: {current_state["session_id"]}\n'
                            )
        result = await session_manager.run_command(current_state['session_id'], message.text)
        if not result:
            await message.reply(f'```\nNothing to show```', parse_mode=ParseMode.MARKDOWN)
            return

        results = chunks(result, 4095)
        for res in results:
            await message.reply(f'```\n{res}```', parse_mode=ParseMode.MARKDOWN)
