import logging

from aiogram.dispatcher import FSMContext
from aiogram.dispatcher.filters.state import State, StatesGroup, default_state
from aiogram.types import Message, ParseMode, InputFile, ChatActions, ContentType

from src.models.server import ServerPermissions
from src.utils.debug_mode import debug_message
from src.utils.decorators import (
    bot_action,
    BotParam
)
from src.utils.server_utils import get_server_by_alias
from src.utils.ssh import download_file, upload_file

logger = logging.getLogger(__name__)


class UploadingStates(StatesGroup):
    start_uploading = State()


@bot_action(r'sys admin scp( help)?$')
async def scp_help(message: Message, state=None):
    await message.reply(
        '```\n'
        '* upload {alias} {path} {user}?- Upload file on server as a user. '
        'If user is null then the file will be uploaded as by root.\n\n'
        '* download {alias} {file_path} {user}? - Download specific file from server as user.\n'
        '```',
        parse_mode=ParseMode.MARKDOWN
    )


@bot_action(r'sys admin scp download',
            params=[
                BotParam('alias', r'[\w\d]+'),
                BotParam('file_path', r'\.?[\/\w\-\d\.]+'),
                BotParam('user', r'(\s+)?\.?[\/\w\-\d\.]+', optional=True)
            ])
async def download(message: Message, alias, file_path, user: str, state=None):
    server = await get_server_by_alias(message, alias, ServerPermissions.INFO)
    await ChatActions.upload_document()
    user = user.strip() if user else None
    filename = file_path.split("/")[-1]
    async with download_file(server, file_path, user) as downloaded_file:
        await message.reply_document(InputFile(downloaded_file, filename=filename), file_path)


@bot_action(r'sys admin scp upload',
            params=[
                BotParam('alias', r'[\w\d]+'),
                BotParam('file_path', r'\.?[\/\w\-\d\.]+'),
                BotParam('user', r'(\s+)?\.?[\/\w\-\d\.]+', optional=True)
            ])
async def upload_start(message: Message, alias: str, file_path: str, user: str, state: FSMContext):
    await debug_message(
        message,
        '```\n'
        f'server: {alias}\n'
        f'file: {file_path}\n'
        f'user: {user.strip() if user else None}\n'
        '```'
    )
    await get_server_by_alias(message, alias, ServerPermissions.SCP)
    await UploadingStates.start_uploading.set()
    async with state.proxy() as current_state:
        current_state['alias'] = alias
        current_state['filename'] = file_path
        current_state['user'] = user.strip() if user else None
    await message.reply(f"Send a file in this chat to upload file on {alias} server")


@bot_action(filter_state=UploadingStates.start_uploading, content_types=ContentType.DOCUMENT)
async def upload(message: Message, state: FSMContext):
    async with state.proxy() as current_state:
        alias = current_state["user"]
        user = current_state["user"]
        filename = current_state["filename"]
        server = await get_server_by_alias(message, current_state['alias'], ServerPermissions.SCP)
        await default_state.set()
        if not message.document:
            await message.reply("This is not a document. Please attach a file.")
            return
        await debug_message(
            message,
            f'server: {alias}\n'
            f'filename: {filename}\n'
            f'user: {user}\n'
        )
        async with upload_file(server, message.document, filename, user) as f:
            await debug_message(message, f"File loaded at {f}")
            await message.reply("File successfully uploaded")
