import logging

from aiogram.types import Message, ParseMode

from src.models.crud.server_crud import (
    fn_create_server,
    fn_get_server,
    fn_delete_server,
    fn_update_server
)
from src.models.server import ServerPermissions, Server
from src.settings import settings
from src.utils.debug_mode import debug_message
from src.utils.decorators import (
    bot_action,
    BotParam
)
from src.utils.server_utils import get_server_by_alias

logger = logging.getLogger(__name__)


@bot_action(f'sys admin servers( help)?$')
async def servers_help(message: Message, state=None):
    await message.reply(
        '```\n'
        '* add {server_address} {alias} - Link server to chat with specific alias\n\n'
        '* rename {old_alias} {new_alias} - Assign new alias to server\n\n'
        '* delete {alias} - Remove server link\n\n'
        '* ls - List all linked servers\n'
        '```',
        parse_mode=ParseMode.MARKDOWN
    )


@bot_action(trigger_str=r'sys admin servers add',
            params=[
                BotParam('address', r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}(\:(?P<port>\d+))?'),
                BotParam('alias', r'[\w\d]+')
            ])
async def add_server(message: Message, address, alias, port, state=None):
    await debug_message(message,
                        f'Chat id: {message.chat.id}\n'
                        f'Address: {address}\n'
                        f'Port: {port}\n'
                        f'Alias: {alias}\n'
                        f'Current user: {str(message.from_user.id)}\n'
                        )

    await fn_create_server(settings.engine,
                           chat_id=str(message.chat.id),
                           server_address=address,
                           server_alias=alias,
                           server_admin=str(message.from_user.id),
                           server_port=port)
    await message.reply(
        f"Server {alias} linked to this chat. Please add text of the next message to your `/root/.ssh/authorized_keys`",
        parse_mode=ParseMode.MARKDOWN
    )
    await settings.dispatcher.bot.send_message(message.chat.id,
                                               f'```\n{settings.public_key}```',
                                               parse_mode=ParseMode.MARKDOWN)


@bot_action(trigger_str=r'sys admin servers rename',
            params=[
                BotParam('old_alias', r'[\w\d]+'),
                BotParam('new_alias', r'[\w\d]+')
            ])
async def rename_server(message: Message, old_alias, new_alias, state=None):
    server = await get_server_by_alias(message, old_alias)
    await debug_message(message,
                        f'Chat id: {message.chat.id}\n'
                        f'Old alias: {old_alias}\n'
                        f'New alias: {new_alias}\n'
                        f'Server admin: {server.server_admin}\n'
                        f'Current user: {str(message.from_user.id)}\n'
                        )
    if server.server_admin != str(message.from_user.id):
        raise Exception(f'You do not have enough permissions to rename this server. \nOnly server admins can do this.')
    await fn_update_server(settings.engine,
                           chat_id=str(message.chat.id),
                           server_alias=old_alias,
                           new_server_alias=new_alias)
    await message.reply(f'Server {old_alias} successfully renamed to {new_alias}')


@bot_action(trigger_str=r'sys admin servers delete',
            params=[
                BotParam('server_alias', r'[\w\d]+')
            ])
async def delete_server(message: Message, server_alias, state=None):
    server = await get_server_by_alias(message, server_alias)
    await debug_message(message,
                        f'Requested server: {server_alias}\n'
                        f'Server permission value: {server.server_permissions.value}'
                        f' ({server.server_permissions.name})\n'
                        f'Server admin: {server.server_admin}\n'
                        f'Current user: {str(message.from_user.id)}\n'
                        )
    if (server.server_admin != str(message.from_user.id) and
            server.server_permissions.value < ServerPermissions.DELETE.value):
        raise Exception(f'You do not have enough permissions to delete this server. \nOnly server admins can do this.')

    await fn_delete_server(settings.engine, chat_id=str(message.chat.id), server_alias=server_alias)
    await message.reply(f'Server `{server_alias}` deleted successfully', parse_mode=ParseMode.MARKDOWN)


@bot_action(trigger_str='sys admin servers ls')
async def list_servers(message: Message, state=None):
    chat_servers = await fn_get_server(settings.engine,
                                       chat_id=str(message.chat.id))

    async def server_repr(server_obj: Server):
        admin = await settings.dispatcher.bot.get_chat_member(message.chat.id, int(server_obj.server_admin))
        return f"```\n" \
               f"address: {server_obj.server_address}\n" \
               f"port: {server_obj.server_port}\n" \
               f"alias: {server_obj.server_alias}\n" \
               f"permission_level: {server_obj.server_permissions.name}\n" \
               f"admin: @{admin.user.username}\n" \
               f"```\n"

    servers_list = []
    for server in chat_servers:
        servers_list.append(await server_repr(server))
    total_number = len(servers_list)
    servers_list = "\n".join(servers_list)
    await debug_message(message,
                        f'Chat id: {message.chat.id}\n'
                        f'Total length: {total_number}\n'
                        f'Current user: {str(message.from_user.id)}\n')
    await message.reply(
        f'All available servers: \nTotal: {total_number} \n\n{servers_list if servers_list else "No servers available"}'
        , parse_mode=ParseMode.MARKDOWN)
