import logging
import re

from aiogram.types import Message, ParseMode

from src.models.server import ServerPermissions
from src.utils.debug_mode import debug_message
from src.utils.decorators import (
    bot_action,
    BotParam
)
from src.utils.server_utils import get_server_by_alias
from src.utils.ssh import run_ssh_command

logger = logging.getLogger(__name__)


@bot_action(r'sys admin sys( help)?$')
async def overall_help(message: Message, state=None):
    await message.reply(
        '```\n'
        '* sys stats {alias} {service} - Check status of the service\n\n'
        '* sys start {alias} {service} - Start the service\n\n'
        '* sys stop {alias} {service} - Stop the service\n\n'
        '* sys restart {alias} {service} - Restart the service\n\n'
        '* sys cat {alias} {service} - Show one or more service unit files\n\n'
        '```',
        parse_mode=ParseMode.MARKDOWN
    )


@bot_action(r'sys admin sys stats',
            params=[
                BotParam('alias', r'[\w\d]+'),
                BotParam('service', r'[\w\d]+\.service')
            ])
async def status(message: Message, alias, service, state):
    server = await get_server_by_alias(message,
                                       server_alias=alias,
                                       minimal_permission=ServerPermissions.SERVICES)
    service_info = await run_ssh_command(server, f'systemctl status {service}')
    description = re.search(r'(.*) - (?P<description>.*)', service_info.splitlines()[0])
    startup_file = re.search(r'(.*)Loaded: loaded \((?P<file>.*\.service)', service_info)
    service_state = re.search(r'(.*)Active: (?P<state>.*) since (.*); (?P<uptime>.*)', service_info)
    pid = re.search(r'(.*)Main PID: (?P<pid>.*)', service_info)
    tasks = re.search(r'(.*)Tasks: (?P<tasks>.*)', service_info)
    memory = re.search(r'(.*)Memory: (?P<memory>.*)', service_info)
    res_message = "```\n"
    await debug_message(message, service_info)
    for regex in [description, startup_file, service_state, pid, tasks, memory]:
        if regex is None:
            continue
        for key, val in regex.groupdict().items():
            res_message += f'{key}: {val}\n\n'
    res_message += '```'
    await message.reply(res_message, parse_mode=ParseMode.MARKDOWN)


@bot_action(r'sys admin sys start',
            params=[
                BotParam('alias', r'[\w\d]+'),
                BotParam('service', r'[\w\d]+\.service')
            ])
async def start(message: Message, alias, service, state):
    server = await get_server_by_alias(message,
                                       server_alias=alias,
                                       minimal_permission=ServerPermissions.SERVICES)
    await run_ssh_command(server, f'systemctl start {service}')
    await message.reply("Service started")


@bot_action(r'sys admin sys stop',
            params=[
                BotParam('alias', r'[\w\d]+'),
                BotParam('service', r'[\w\d]+\.service')
            ])
async def stop(message: Message, alias, service, state):
    server = await get_server_by_alias(message,
                                       server_alias=alias,
                                       minimal_permission=ServerPermissions.SERVICES)
    await run_ssh_command(server, f'systemctl stop {service}')
    await message.reply("Service stopped")


@bot_action(r'sys admin sys restart',
            params=[
                BotParam('alias', r'[\w\d]+'),
                BotParam('service', r'[\w\d]+\.service')
            ])
async def restart(message: Message, alias, service, state):
    server = await get_server_by_alias(message,
                                       server_alias=alias,
                                       minimal_permission=ServerPermissions.SERVICES)
    await run_ssh_command(server, f'systemctl restart {service}')
    await message.reply("Service restarted")


@bot_action(r'sys admin sys cat',
            params=[
                BotParam('alias', r'[\w\d]+'),
                BotParam('service', r'[\w\d]+\.service')
            ])
async def cat(message: Message, alias, service, state):
    server = await get_server_by_alias(message,
                                       server_alias=alias,
                                       minimal_permission=ServerPermissions.SERVICES)
    out = await run_ssh_command(server, f'systemctl cat {service}')
    await message.reply(
        '```\n'
        f'{out}'
        '```',
        parse_mode=ParseMode.MARKDOWN
    )
