import logging
from typing import Optional, List

from aiopg.sa import Engine
from sqlalchemy import select

from src.models import Server
from src.models.server import ServerPermissions

logger = logging.getLogger()


async def fn_create_server(engine: Engine, *,
                           chat_id: str,
                           server_address: str,
                           server_admin: str,
                           server_alias: str,
                           server_port: Optional[int] = None) -> None:
    insert_query = Server.__table__.insert().values(
        chat_id=chat_id,
        server_alias=server_alias,
        server_address=server_address,
        server_admin=server_admin,
        server_port=server_port if server_port else 22
    )
    async with engine.acquire() as db_connection:
        await db_connection.execute(insert_query)


async def fn_get_server(engine: Engine, *,
                        chat_id: Optional[str] = None,
                        server_address: Optional[str] = None,
                        server_alias: Optional[str] = None) -> List[Server]:
    select_query = Server.__table__.select()
    if chat_id:
        select_query = select_query.where(Server.chat_id == chat_id)
    if server_address:
        select_query = select_query.where(Server.server_address == server_address)
    if server_alias:
        select_query = select_query.where(Server.server_alias == server_alias)
    async with engine.acquire() as db_connection:
        cursor = await db_connection.execute(select_query)
        return await cursor.fetchall()


async def fn_update_server(engine: Engine, *,
                           chat_id: str,
                           server_alias: str,
                           new_server_alias: Optional[str] = None,
                           new_server_permissions: Optional[ServerPermissions] = None,
                           new_server_address: Optional[str] = None):
    update_values = {}
    if new_server_address:
        update_values['server_address'] = new_server_address
    if new_server_alias:
        update_values['server_alias'] = new_server_alias
    if new_server_permissions:
        update_values['server_permissions'] = new_server_permissions
    update_query = (Server.__table__
                    .update()
                    .where(Server.chat_id == chat_id)
                    .where(Server.server_alias == server_alias)
                    .values(update_values))
    async with engine.acquire() as db_connection:
        await db_connection.execute(update_query)


async def fn_delete_server(engine: Engine, *,
                           chat_id: str,
                           server_alias: str):
    delete_query = (Server.__table__
                    .delete()
                    .where(Server.chat_id == chat_id)
                    .where(Server.server_alias == server_alias))
    async with engine.acquire() as db_connection:
        await db_connection.execute(delete_query)


async def fn_get_all_chat_ids(engine: Engine) -> List[int]:
    chat_select = select([Server.chat_id]).group_by(Server.chat_id).distinct()
    async with engine.acquire() as connection:
        cursor = await connection.execute(chat_select)
        servers = await cursor.fetchall()
    return list(map(lambda x: int(x.chat_id), servers))
