import logging
import re
from functools import wraps
from typing import Callable, List, Tuple

from aiogram.dispatcher.filters.state import State
from aiogram.types import Message, ParseMode, ReplyKeyboardRemove

from src.settings import settings

logger = logging.getLogger(__name__)


class BotParam(object):
    def __init__(self, name, regex, optional=False):
        self.name = name
        self.regex = regex
        self.optional = optional

    def get_regex(self):
        option = "?" if self.optional else ""
        sep = "" if self.optional else " "
        return f"{sep}(?P<{self.name}>{self.regex}){option}"


def cool_response_exception(f: Callable):
    """
    Wraps message handler to reply to user with
    user-friendly message instead of ignoring.

    :param f: wrapped function
    :return: Any
    """

    @wraps(f)
    async def decorated_func(message: Message, state=None):
        try:
            result = await f(message, state)
            return result
        except Exception as e:
            await message.reply(f"Command cannot be executed, because of the following error: \n```\n{str(e)}```",
                                parse_mode=ParseMode.MARKDOWN,
                                reply_markup=ReplyKeyboardRemove())
            logger.exception(e)

    return decorated_func


def bot_action(trigger_str: str = None, params: List[Tuple] = None, filter_state: State = None,
               content_types=None) -> Callable:
    """
    Mark function as bot message handler.
    params -> is just an array of tuples.
    Each value have regex string and name


    :param trigger_str: message that triggers bot
    :param params: parameters in message
    :param filter_state: trigger state if you use Finite State Machine mechanism
    :param content_types: Which content type bot is waiting for
    :return: decorator
    """

    def decor(f: Callable):
        regexp_params = ''.join(map(lambda x: x.get_regex(), params)) if params else ''
        filter_regexp = f'^{trigger_str}{regexp_params}'.strip()

        @wraps(f)
        @settings.dispatcher.message_handler(regexp=filter_regexp if trigger_str else None,
                                             state=filter_state,
                                             content_types=content_types)
        @cool_response_exception
        async def modified_function(message: Message, state=None):
            additional_values = {}
            if params:
                match = re.match(filter_regexp, message.text)
                additional_values = match.groupdict()
            if trigger_str:
                message.text = message.text.replace(trigger_str, '', 1).strip()

            result = await f(message=message, state=state, **additional_values)
            return result

        return modified_function

    return decor
