import asyncio
import logging
import os
import uuid
from contextlib import asynccontextmanager

import asyncssh
from aiogram.types import Document

from src.models import Server

logger = logging.getLogger(__name__)


async def open_ssh_session(server: Server) -> asyncssh.SSHClientProcess:
    """
    Open bash process over ssh.

    :param server: server where pipe needs to be opened
    :return: opened process
    """
    connection = await asyncssh.connect(server.server_address, server.server_port, known_hosts=None)
    process = await connection.create_process('/bin/bash')
    return process


async def collect_output(out_pipe, timeout):
    """
    Collect output from command.
    output will be collected from out_pipe.

    :param out_pipe: async io channel
    :param timeout:  timeout to read from channel
    :return:
    """
    res = []
    try:
        line = await asyncio.wait_for(out_pipe.readline(), timeout)
        while line:
            res.append(await asyncio.wait_for(out_pipe.readline(), timeout))
    except asyncio.exceptions.TimeoutError:
        return ''.join(res).strip()
    return ''.join(res).strip()


async def run_interactive_command(command: str,
                                  process: asyncssh.SSHClientProcess,
                                  timeout=0.5) -> str:
    """
    Run command on running ssh session.
    Command will be executed on server with provided process.
    Timeout used to collect output from commands.

    :param command:  command yo execute
    :param process:  where ot execute
    :param timeout:  receiving  timeout
    :return:stdout or stderr as exception
    """
    process.stdin.write(command + '\n')
    stdout = await collect_output(process.stdout, timeout)
    stderr = await collect_output(process.stderr, timeout)
    if stderr:
        raise Exception(stderr)
    return stdout


async def run_ssh_command(server: Server, command: str) -> str:
    """
    Run command on remote server.
    Passed command will be executed as root.

    :param server: remote server
    :param command: command to execute
    :return: output.
    """
    process = await open_ssh_session(server)
    res = await run_interactive_command(command, process)
    process.close()
    return res


@asynccontextmanager
async def download_file(server: Server, file_path: str, username: str) -> str:
    """
    Download file from server and store it as random file in tmp.
    When context is over, than it will remove file from tmp.

    :param server: remote server.
    :param file_path: file path on remote server
    :param username: as who you want to log in
    :return: path to downloaded file
    """
    load_path = f'/tmp/{uuid.uuid4()}'
    async with asyncssh.connect(server.server_address,
                                server.server_port,
                                known_hosts=None,
                                username=username) as conn:
        await asyncssh.scp((conn, file_path), load_path)
    yield load_path
    os.remove(load_path)


@asynccontextmanager
async def upload_file(server: Server, document: Document, file_path: str, username: str):
    """
    Upload message document to a server.

    :param server: server where to upload
    :param document: amessage attached document
    :param file_path: remote file path to store file
    :param username:  user to login as
    :return:
    """
    load_path = f'/tmp/{uuid.uuid4()}'
    await document.download(load_path)
    async with asyncssh.connect(server.server_address,
                                server.server_port,
                                known_hosts=None,
                                username=username) as conn:
        await asyncssh.scp(load_path, (conn, file_path))
    yield load_path
    os.remove(load_path)


class SessionManager(object):
    """
    Class to manage all connection to other servers over ssh.
    """
    def __init__(self):
        self.__connections = dict()

    async def add_connection(self, server: Server) -> str:
        """
        Add connection some server and assign uuid to it.
        This method will open chanel and return uuid of session.

        :param server: server to open connection at
        :return: valid uuid4 string
        """
        rand_uuid = str(uuid.uuid4())
        session = await open_ssh_session(server)
        self.__connections[rand_uuid] = session
        return rand_uuid

    async def run_command(self, connection_id: str, command: str):
        """
        Run command in opened session by id.

        :param connection_id: valid uuid of opened session
        :param command: command to run over ssh
        :return: command output
        """
        return await run_interactive_command(command, self.__connections[connection_id])

    def close(self, connection_id: str):
        self.__connections[connection_id].close()


session_manager = SessionManager()
